import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution

class Binomial(Distribution):
    '''
        Binomial distribution class for calculating and visualizing a Binomial distribution.

        Attributes:
            mean (float) representing the mean value of the distribution
            stdev (float) representing the standard deviation of the distribution
            data_list (list of floats) a list of floats to be extracted from the data file
            p (float) representing the probability of an event occurring
            n (int) the total number of trials
    '''

    def __init__(self, prob=0.5, size=20):
        self.p = prob
        self.n = size
        Distribution.__init__(self, self.calculate_mean(), self.calculate_stdev())

    def calculate_mean(self):
        '''
            Function to calculate the mean from p and n

            Args:
                None

            Returns:
                float: mean of the data set
        '''

        self.mean = self.n * self.p
        return self.mean

    def calculate_stdev(self):
        '''
            Function to calculate the standard deviation from p and n.

            Arg:
                None

            Returns:
                float: standard deviation of the data set
        '''

        self.stdev = math.sqrt(self.n * self.p *(1 - self.p))
        return self.stdev

    def replace_stats_with_data(self):
        '''
            Function to calculate p and n from the data set

            Args:
                None

            Returns:
                float: the p value
                float: the n value
        '''

        self.n = len(self.data)
        self.p = sum(self.data)/len(self.data)
        self.calculate_mean()
        self.calculate_stdev()

        return self.p, self.n

    def plot_bar(self):
        '''
            Function to output a histogram of the instance variable data using matplotlib pyplot library.

            Args:
                None

            Returns:
                None
        '''

        plt.bar(x=set(self.data), height=[self.data.count(num) for num in list(set(self.data))], width=0.8);
        plt.title('Bar chart of the frequency of success and failure');
        plt.xlabel('Discrete states');
        plt.ylabel('Count');
        plt.show()

    def pdf(self, k):
        '''
            Probability density function calculator for the Gaussian distribution.

            Args:
                k (float): point for calculating the probability density function

            Returns:
                float: probability density function output
        '''

        a = (math.factorial(self.n))/(math.factorial(k) * (math.factorial(self.n - k)))
        b = math.pow(self.p, k) * (math.pow((1 - self.p), (self.n - k)))
        return a * b

    def plot_bar_pdf(self):
        '''
            Function to plot the pdf of the binomial distribution

            Args:
                None

            Returns:
                list: x values for the pdf plot
                list: y values for the pdf plot
        '''

        x = list(range(self.n + 1))
        y = [self.pdf(num) for num in x]
        plt.bar(x, y)
        plt.title('Bar chart of the Binomial Distribution')
        plt.xlabel('Number of successes (k)')
        plt.ylabel('Probability of success')

        plt.show()

        return x, y

    def __add__(self, other):
        '''
            Function to add together two Binomial distributions with equal p

            Args:
                other (Binomial): Binomial instance

            Returns:
                Binomial: Binomial distribution
        '''

        try:
            assert self.p == other.p, 'p values are not equal'
        except AssertionError as error:
            raise

        result = Binomial()
        result.n = self.n + other.n
        result.p = self.p
        result.calculate_mean()
        result.calculate_stdev()

        return result

    def __repr__(self):
        '''
            Function to output the characteristics of the Binomial instance

            Args:
                None

            Returns:
                string: characteristics of the Gaussian
        '''

        return 'mean {}, standard deviation {}, p {}, n {}'.format(self.mean, self.stdev, self.p, self.n)


import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution

class Gaussian(Distribution):
    '''
        Gaussian distribution class for calculating and visualizing a Gaussian distribution.

        Attributes:
            mean (float) representing the mean value of the distribution
            stdev (float) representing the standard deviation of the distribution
            data_list (list of floats) a list of floats extracted from the data file
    '''

    def __init__(self, mu=0, sigma=1):
        Distribution.__init__(self, mu, sigma)

    def calculate_mean(self):
        '''
            Function to calculate the mean of the data set.

            Args:
                None
            Returns:
                float: mean of the data set
        '''

        avg = 1.0 * sum(self.data) / len(self.data)
        self.mean = avg
        return self.mean

    def calculate_stdev(self, sample=True):
        '''
            Function to calculate the standard deviation of the data set.

            Args:
                sample (bool): whether the data represents a sample of population
            Returns:
                float: standard deviation of the data set
        '''

        if sample:
            n = len(self.data) - 1
        else:
            n = len(self.data)

        mean = self.calculate_mean()
        sigma = 0

        for d in self.data:
            sigma += (d - mean) ** 2

        sigma = math.sqrt(sigma / n)
        self.stdev = sigma

        return self.stdev

    def plot_histogram(self):
        '''
            Function to output a histogram of the instance variable data using matplotlib pyplot library.

            Args:
                None
            Returns:
                None
        '''

        plt.hist(self.data)
        plt.title('Histogram of Data')
        plt.xlabel('data')
        plt.ylabel('count')

    def pdf(self, x):
        '''
            Probability density function calculator for the Gaussian distribution

            Args:
                x (float): point for calculating the probability density function
            Returns:
                float: probability density function output
        '''

        return (1.0 / (self.stdev * math.sqrt(2*math.pi))) * math.exp(-0.5*((x - self.mean) / self.stdev) ** 2)

    def plot_histogram_pdf(self, n_spaces = 50):
        '''
            Function to plot the normalized histogram of data and a plot of the probability 
            density function along the same range

            Args:
                n_spaces (int): number of data points
            Returns:
                list: x values for the pdf plot
                list: y values for the pdf plot
        '''

        mu = self.mean
        sigma = self.stdev
        min_range = min(self.data)
        max_range = max(self.data)

        #calculates the interval between x values

        interval = 1.0 * (max_range - min_range) / n_spaces

        x = []
        y = []

        #calculates the interval between x values

        for i in range(n_spaces):
            tmp = min_range + interval*i
            x.append(tmp)
            y.append(self.pdf(tmp))

        #make the plots
        fig, axes = plt.subplots(2, sharex=True)
        fig.subplots_adjust(hspace=0.5)
        axes[0].hist(self.data, density=True)
        axes[0].set_title('Normed Histogram of Data')
        axes[0].set_ylabel('Density')

        axes[1].plot(x,y)
        axes[1].set_title('Normal Distribution for Sample Mean and Sample Standard Deviation')

        axes[1].set_ylabel('Density')
        plt.show()

        return x, y

    def __add__(self, other):
        '''
            Function to add together two Gaussian distributions

            Args:
                other (Gaussian): Gaussian instance
            Returns:
                Gaussian: Gaussian distribution
        '''

        result = Gaussian()
        result.mean = self.mean + other.mean
        result.stdev = math.sqrt(self.stdev ** 2 + other.stdev ** 2)

        return result

    def __repr__(self):
        '''
            Function to output the characteristics of the Gaussian instance

            Args:
                None
            Returns:
                string: characteristics of the Gaussian
        '''

        return 'mean {}, standard deviation {}'.format(self.mean, self.stdev)

    def read_data_file(self, file_name, sample = True):
        '''
            Function to read in data from a txt file. The txt file should have one
            number (float) per line. The numbers are stored in the data attribute.

            After reading the file, the mean and standard deviation are calculated

            Args:
                file_name (string): name of a file to read from
            Returns:
                None
        '''

        with open(file_name) as file:
            data_list = []
            line = file.readline()
            while line:
                data_list.append(int(line))
                line = file.readline()
        file.close()

        self.data = data_list
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev(sample)


from setuptools import setup

setup(name='Gaus_dotgb_Bin',
      version='0.1',
      description='Gaussian and Binomial distributions',
      packages=['Gaus_dotgb_Bin'],
      author = 'dotgb',
      author_email = 'myemail@something.com',
      zip_safe=False)

